!*************************************
!usage:
!./rotated.exe INFILE OUTFILE1 OUTFILE2
!eg:
!./rotated.exe 2015033018_natural_glo.txt 2015033018_rotated_reg.txt 2015033018_natural_reg.txt
!*************************************
!compile with:
!gfortran rotated.f90 -o rotated.exe
!*************************************
!file to be included:
!modelgrid_0101.inc it contains the regional domain definition
!*************************************
!INFILE is an ASCII file with global observations in natural lon/lat, with format:
!number of obs
!date,lon,lat,value,uncertainty
!*************************************
!OUTFILE1 is an ASCII file with regional observations in rotatated lon/lat, with format:
!date,lon,lat,value,uncertainty
!*************************************
!OUTFILE2 is an ASCII file with regional observations in natural lon/lat, with format:
!date,lon,lat,value,uncertainty
!*************************************

program rotated

implicit none
include 'modelgrid_0101.inc'

real(kind=4):: tph0,ctph0,stph0,xlon,ylat,tlm,tph, REAL_I,REAL_J,rlon,rlat
real:: val,unc, bcw,bcs
integer(kind=4):: obsout,nobs,idate,i
character*164:: infile,outfile,out2file
real(kind=4),parameter:: &
 pi=3.141592653589793238 &   !
,dtr=pi/180.             &   ! from degree to rad
,rtd=180./pi                 ! from rad to degree


tph0=tph0d*dtr !rad of the centre lat
ctph0=cos(tph0) !cos of the centre lat
stph0=sin(tph0) !sin of the centre lat

call getarg(1, infile)
call getarg(2, outfile) 
call getarg(3, out2file)

!file with original global observations
!with format:
!number of obs
!date,lon,lat,value,uncertainty
open(unit=1,file=infile,status='old',form='formatted')
!file with rotatated observations inside the regional domain
open(unit=2,file=outfile,form='formatted')
!file with natural observations inside the regional domain (to plot them)
open(unit=3,file=out2file,form='formatted')

!if you want to screen out obs close to the boundary
!bcw=abs(wbd)-2.
!bcs=abs(sbd)-2.
!otherwise:
bcw=abs(wbd)
bcs=abs(sbd)

read(1,*) nobs
do i=1,nobs
    read(1,*) idate,xlon,ylat,val,unc

    !calculate rotated coordinates
    call tll(xlon,ylat,tlm0d,dtr,ctph0,stph0,tlm,tph)

    rlon=tlm*rtd
    rlat=tph*rtd
 
    !select obs inside the domain only   
    obsout=1

    if ((abs(rlon).lt. bcw).and.(abs(rlat).lt.bcs))then
       obsout=0
    endif

    if (obsout.eq.0) then
      !'keep obs'
      write(2,'(I10,1X,F7.3,1X,F7.3,1X,F8.4,1X,F8.4)') idate,rlon,rlat,val,unc
      write(3,'(I10,1X,F7.3,1X,F7.3,1X,F8.4,1X,F8.4)') idate,xlon,ylat,val,unc
    endif 

enddo
close(1)
close(2)
close(3)

!call CONVERT_LATLON_TO_IJ(ylat,xlon,TPH0_1,TLM0_1,SB_1,WB_1,RECIP_DPH_1,RECIP_DLM_1,REAL_I,REAL_J)

!-----------------------------------------------------------------------
endprogram rotated
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine tll(xlon,xlat,tlm0d,dtr,ctph0,stph0,tlm,tph)
!     ****************************************************************
!     *                                                              *
!     *  programer: z. janjic, shmz, feb. 1981                       *
!     *  ammended:  z. janjic, ncep, jan. 1996                       *
!     *                                                              *
!     *  transformation from lat-lon to rotated lat-lon coordinates  *
!     ****************************************************************
!     ****************************************************************
!     *  tlm   - transformed longitude, rad.                         *
!     *  tph   - transformed latitude, rad.                          *
!     *  tlm0d - the angle of rotation of the transformed lat-lon    *
!     *          system in the longitudinal direction, degs          *
!     *  ctph0 - cos(tph0), tph0 is the angle of rotation of the     *
!     *          transformed lat-lon systemn in the latitudinal      *
!     *          direction, precomputed                              *
!     *  stph0 - sin(tph0), tph0 is the angle of rotation of the     *
!     *          transformed lat-lon systemn in the latitudinal      *
!     *          direction, precomputed                              *
!     *  almd  - geographical longitude, degs, range -180.,180       *
!     *  aphd  - geographical latitude,  degs, range - 90., 90.,     *
!     *          poles are singular                                  *
!     ****************************************************************
!
      relm=(xlon-tlm0d)*dtr !distance from the centre lon (in rad)
      srlm=sin(relm) !sin of this distance
      crlm=cos(relm) !cos of this distance
      aph=xlat*dtr !lat in rad 
      sph=sin(aph) !sin of lat
      cph=cos(aph) !cos of lat
      cc=cph*crlm !cos of lat times cos of lon distance
      anum=cph*srlm !cos of lat times sin of lon distance
      denom=ctph0*cc+stph0*sph !cos of the centre lat times cc plus sin of the centre lat times sin of lat
!
      tlm=atan2(anum,denom)
      tph=asin(ctph0*sph-stph0*cc)
!
      return
      endsubroutine tll
!-----------------------------------------------------------------------

!#######################################################################
!-----------------------------------------------------------------------
!
      SUBROUTINE CONVERT_LATLON_TO_IJ(GLAT,GLON                         &
                             ,TPH0_1,TLM0_1                             &
                             ,SB_1,WB_1                                 &
                             ,RECIP_DPH_1,RECIP_DLM_1                   &
                             ,REAL_I,REAL_J)
! 
!-----------------------------------------------------------------------
!***  Determine the Real values of I and J on a rotated B Grid given
!***  the geographic latitude/longitude.  In this routine the domain
!***  is specifically that of the uppermost parent.
!-----------------------------------------------------------------------
!
      INTEGER, PARAMETER :: DOUBLE=SELECTED_REAL_KIND(P=13,R=200)
!
!------------------------
!***  Argument Variables
!------------------------
!
      REAL,INTENT(IN) :: GLAT                                &  !<-- Geographic latitude (radians, positive north)
                                   ,GLON                                &  !<-- Geographic longitude (radians, positive east)
                                   ,TPH0_1                              &  !<-- Central latitude of uppermost parent (radians, north)
                                   ,TLM0_1                              &  !<-- Central longitude of uppermost parent (radians, east)
                                   ,SB_1                                &  !<-- Rotated lat of upper parent's south boundary (radians, north)
                                   ,WB_1                                &  !<-- Rotated lon of upper parent's west boundary (radians, east)
                                   ,RECIP_DPH_1                         &  !<-- Reciprocal of upper parent's grid increment (radians) in J
                                   ,RECIP_DLM_1                            !<-- Reciprocal of upper parent's grid increment (radians) in I
!
      REAL,INTENT(OUT) :: REAL_I                             &  !<-- Real I on uppermost parent grid for GLAT,GLON
                                    ,REAL_J                                !<-- Real J on uppermost parent grid for GLAT,GLON
!
!---------------------
!***  Local Variables
!---------------------
!
      REAL :: TLAT,TLON                                      &
                        ,X,Y,Z
!
!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
!
      X=COS(TPH0_1)*COS(GLAT)*COS(GLON-TLM0_1)+SIN(TPH0_1)*SIN(GLAT)
      Y=COS(GLAT)*SIN(GLON-TLM0_1)
      Z=COS(TPH0_1)*SIN(GLAT)-SIN(TPH0_1)*COS(GLAT)*COS(GLON-TLM0_1)
      TLAT=ATAN(Z/SQRT(X*X+Y*Y))
      TLON=ATAN(Y/X)
!
!     REAL_I=(TLON-WB_1)*RECIP_DLM_1+1
!     REAL_J=(TLAT-SB_1)*RECIP_DPH_1+1
      REAL_I=(TLON-WB_1)/RECIP_DLM_1+1
      REAL_J=(TLAT-SB_1)/RECIP_DPH_1+1
!
!-----------------------------------------------------------------------
!
      END SUBROUTINE CONVERT_LATLON_TO_IJ
!
!-----------------------------------------------------------------------
!#######################################################################
